@@ -38,7 +38,6 @@
 #include "m_option.h"
 #include "common/common.h"
 #include "stream/stream.h"
-#include "stream/tv.h"
 #include "video/csputils.h"
 #include "sub/osd.h"
 #include "audio/mixer.h"
@@ -46,6 +45,7 @@
 #include "audio/decode/dec_audio.h"
 #include "player/core.h"
 #include "player/command.h"
+#include "stream/stream.h"
 
 extern const char mp_help_text[];
 
@@ -63,6 +63,7 @@ extern const struct m_sub_options tv_params_conf;
 extern const struct m_sub_options stream_pvr_conf;
 extern const struct m_sub_options stream_cdda_conf;
 extern const struct m_sub_options stream_dvb_conf;
+extern const struct m_sub_options stream_lavf_conf;
 extern const struct m_sub_options sws_conf;
 extern const struct m_sub_options demux_rawaudio_conf;
 extern const struct m_sub_options demux_rawvideo_conf;
@@ -83,13 +84,13 @@ extern const struct m_obj_list ao_obj_list;
 const m_option_t mp_opts[] = {
     // handled in command line pre-parser (parse_commandline.c)
     {"v", CONF_TYPE_STORE, CONF_GLOBAL | CONF_NOCFG, .offset = -1},
-    {"playlist", CONF_TYPE_STRING, CONF_NOCFG | M_OPT_MIN | M_OPT_FIXED,
+    {"playlist", CONF_TYPE_STRING, CONF_NOCFG | M_OPT_MIN | M_OPT_FIXED | M_OPT_FILE,
      .min = 1, .offset = -1},
     {"{", CONF_TYPE_STORE, CONF_NOCFG | M_OPT_FIXED, .offset = -1},
     {"}", CONF_TYPE_STORE, CONF_NOCFG | M_OPT_FIXED, .offset = -1},
 
     // handled in m_config.c
-    { "include", CONF_TYPE_STRING, M_OPT_FIXED, .offset = -1},
+    { "include", CONF_TYPE_STRING, M_OPT_FIXED | M_OPT_FILE, .offset = -1},
     { "profile", CONF_TYPE_STRING_LIST, M_OPT_FIXED, .offset = -1},
     { "show-profile", CONF_TYPE_STRING, CONF_NOCFG | M_OPT_FIXED, .offset = -1},
     { "list-options", CONF_TYPE_STORE, CONF_NOCFG | M_OPT_FIXED, .offset = -1},
@@ -126,9 +127,11 @@ const m_option_t mp_opts[] = {
     OPT_STRINGLIST("reset-on-next-file", reset_options, M_OPT_GLOBAL),
 
 #if HAVE_LUA
-    OPT_STRINGLIST("lua", lua_files, CONF_GLOBAL),
+    OPT_STRINGLIST("lua", lua_files, CONF_GLOBAL | M_OPT_FILE),
     OPT_KEYVALUELIST("lua-opts", lua_opts, M_OPT_GLOBAL),
     OPT_FLAG("osc", lua_load_osc, CONF_GLOBAL),
+    OPT_FLAG("ytdl", lua_load_ytdl, CONF_GLOBAL),
+    OPT_STRING("ytdl-format", lua_ytdl_format, CONF_GLOBAL),
     OPT_FLAG("load-scripts", auto_load_scripts, CONF_GLOBAL),
 #endif
 
@@ -141,14 +144,11 @@ const m_option_t mp_opts[] = {
                       ({"no", 0})),
     OPT_INTRANGE("cache-initial", stream_cache.initial, 0, 0, 0x7fffffff),
     OPT_INTRANGE("cache-seek-min", stream_cache.seek_min, 0, 0, 0x7fffffff),
-    OPT_STRING("cache-file", stream_cache.file, 0),
+    OPT_STRING("cache-file", stream_cache.file, M_OPT_FILE),
     OPT_INTRANGE("cache-file-size", stream_cache.file_max, 0, 0, 0x7fffffff),
-    OPT_CHOICE_OR_INT("cache-pause-below", stream_cache_pause, 0, 0, 0x7fffffff,
-                      ({"no", 0})),
-    OPT_INTRANGE("cache-pause-restart", stream_cache_unpause, 0, 0, 0x7fffffff),
 
 #if HAVE_DVDREAD || HAVE_DVDNAV
-    OPT_STRING("dvd-device", dvd_device, 0),
+    OPT_STRING("dvd-device", dvd_device, M_OPT_FILE),
     OPT_INT("dvd-speed", dvd_speed, 0),
     OPT_INTRANGE("dvd-angle", dvd_angle, 0, 1, 99),
 #endif /* HAVE_DVDREAD */
@@ -156,7 +156,7 @@ const m_option_t mp_opts[] = {
     OPT_CHOICE_OR_INT("edition", edition_id, 0, 0, 8190,
                       ({"auto", -1})),
 #if HAVE_LIBBLURAY
-    OPT_STRING("bluray-device", bluray_device, 0),
+    OPT_STRING("bluray-device", bluray_device, M_OPT_FILE),
     OPT_INTRANGE("bluray-angle", bluray_angle, 0, 0, 999),
 #endif /* HAVE_LIBBLURAY */
 
@@ -164,14 +164,14 @@ const m_option_t mp_opts[] = {
     OPT_STRING("user-agent", network_useragent, 0),
     OPT_STRING("referrer", network_referrer, 0),
     OPT_FLAG("cookies", network_cookies_enabled, 0),
-    OPT_STRING("cookies-file", network_cookies_file, 0),
+    OPT_STRING("cookies-file", network_cookies_file, M_OPT_FILE),
     OPT_CHOICE("rtsp-transport", network_rtsp_transport, 0,
                ({"lavf", 0},
                 {"udp", 1},
                 {"tcp", 2},
                 {"http", 3})),
     OPT_FLAG("tls-verify", network_tls_verify, 0),
-    OPT_STRING("tls-ca-file", network_tls_ca_file, 0),
+    OPT_STRING("tls-ca-file", network_tls_ca_file, M_OPT_FILE),
 
 // ------------------------- demuxer options --------------------
 
@@ -182,6 +182,9 @@ const m_option_t mp_opts[] = {
     OPT_REL_TIME("end", play_end, 0),
     OPT_REL_TIME("length", play_length, 0),
 
+    OPT_TIME("ab-loop-a", ab_loop[0], 0, .min = MP_NOPTS_VALUE),
+    OPT_TIME("ab-loop-b", ab_loop[1], 0, .min = MP_NOPTS_VALUE),
+
     OPT_FLAG("pause", pause, M_OPT_FIXED),
     OPT_FLAG("keep-open", keep_open, 0),
 
@@ -192,6 +195,9 @@ const m_option_t mp_opts[] = {
     OPT_TRACKCHOICE("vid", video_id),
     OPT_TRACKCHOICE("sid", sub_id),
     OPT_TRACKCHOICE("secondary-sid", sub2_id),
+    OPT_TRACKCHOICE("ff-aid", audio_id_ff),
+    OPT_TRACKCHOICE("ff-vid", video_id_ff),
+    OPT_TRACKCHOICE("ff-sid", sub_id_ff),
     OPT_FLAG_STORE("no-sub", sub_id, 0, -2),
     OPT_FLAG_STORE("no-video", video_id, 0, -2),
     OPT_FLAG_STORE("no-audio", audio_id, 0, -2),
@@ -201,19 +207,26 @@ const m_option_t mp_opts[] = {
     OPT_CHOICE("audio-display", audio_display, 0,
                ({"no", 0}, {"attachment", 1})),
 
-    OPT_STRING("quvi-format", quvi_format, 0),
-    OPT_FLAG("quvi-fetch-subtitles", quvi_fetch_subtitles, 0),
+    OPT_CHOICE("hls-bitrate", hls_bitrate, M_OPT_FIXED,
+               ({"no", 0}, {"min", 1}, {"max", 2})),
 
 #if HAVE_CDDA
     OPT_SUBSTRUCT("cdda", stream_cdda_opts, stream_cdda_conf, 0),
-    OPT_STRING("cdrom-device", cdrom_device, 0),
+    OPT_STRING("cdrom-device", cdrom_device, M_OPT_FILE),
 #endif
 
     // demuxer.c - select audio/sub file/demuxer
-    OPT_STRING_APPEND_LIST("audio-file", audio_files, 0),
+    OPT_STRING_APPEND_LIST("audio-file", audio_files, M_OPT_FILE),
     OPT_STRING("demuxer", demuxer_name, 0),
     OPT_STRING("audio-demuxer", audio_demuxer_name, 0),
     OPT_STRING("sub-demuxer", sub_demuxer_name, 0),
+    OPT_FLAG("demuxer-thread", demuxer_thread, 0),
+    OPT_DOUBLE("demuxer-readahead-secs", demuxer_min_secs, M_OPT_MIN, .min = 0),
+    OPT_INTRANGE("demuxer-readahead-packets", demuxer_min_packs, 0, 0, MAX_PACKS),
+    OPT_INTRANGE("demuxer-readahead-bytes", demuxer_min_bytes, 0, 0, MAX_PACK_BYTES),
+
+    OPT_DOUBLE("cache-secs", demuxer_min_secs_cache, M_OPT_MIN, .min = 0),
+    OPT_FLAG("cache-pause", cache_pausing, 0),
 
     OPT_DOUBLE("mf-fps", mf_fps, 0),
     OPT_STRING("mf-type", mf_type, 0),
@@ -226,6 +239,7 @@ const m_option_t mp_opts[] = {
 #if HAVE_DVBIN
     OPT_SUBSTRUCT("dvbin", stream_dvb_opts, stream_dvb_conf, 0),
 #endif
+    OPT_SUBSTRUCT("", stream_lavf_opts, stream_lavf_conf, 0),
 
 // ------------------------- a-v sync options --------------------
 
@@ -240,6 +254,8 @@ const m_option_t mp_opts[] = {
     OPT_DOUBLE("speed", playback_speed, M_OPT_RANGE | M_OPT_FIXED,
                .min = 0.01, .max = 100.0),
 
+    OPT_FLAG("audio-pitch-correction", pitch_correction, 0),
+
     // set a-v distance
     OPT_FLOATRANGE("audio-delay", audio_delay, 0, -100.0, 100.0),
 
@@ -267,7 +283,8 @@ const m_option_t mp_opts[] = {
                 {"vdpau", 1},
                 {"vda", 2},
                 {"vaapi", 4},
-                {"vaapi-copy", 5})),
+                {"vaapi-copy", 5},
+                {"dxva2-copy", 6})),
     OPT_STRING("hwdec-codecs", hwdec_codecs, 0),
 
     OPT_SUBSTRUCT("sws", vo.sws_opts, sws_conf, 0),
@@ -289,10 +306,13 @@ const m_option_t mp_opts[] = {
 
     OPT_FLAG("demuxer-mkv-subtitle-preroll", mkv_subtitle_preroll, 0),
     OPT_FLAG("mkv-subtitle-preroll", mkv_subtitle_preroll, 0), // old alias
+    OPT_DOUBLE("demuxer-mkv-subtitle-preroll-secs", mkv_subtitle_preroll_secs,
+               M_OPT_MIN, .min = 0),
+    OPT_FLAG("demuxer-mkv-probe-video-duration", mkv_probe_duration, 0),
 
 // ------------------------- subtitles options --------------------
 
-    OPT_STRING_APPEND_LIST("sub-file", sub_name, 0),
+    OPT_STRING_APPEND_LIST("sub-file", sub_name, M_OPT_FILE),
     OPT_PATHLIST("sub-paths", sub_paths, 0),
     OPT_STRING("sub-codepage", sub_cp, 0),
     OPT_FLOAT("sub-delay", sub_delay, 0),
@@ -317,13 +337,13 @@ const m_option_t mp_opts[] = {
     OPT_FLAG("ass-vsfilter-blur-compat", ass_vsfilter_blur_compat, 0),
     OPT_FLAG("embeddedfonts", use_embedded_fonts, 0),
     OPT_STRINGLIST("ass-force-style", ass_force_style_list, 0),
-    OPT_STRING("ass-styles", ass_styles_file, 0),
+    OPT_STRING("ass-styles", ass_styles_file, M_OPT_FILE),
     OPT_CHOICE("ass-hinting", ass_hinting, 0,
                ({"none", 0}, {"light", 1}, {"normal", 2}, {"native", 3})),
     OPT_CHOICE("ass-shaper", ass_shaper, 0,
                ({"simple", 0}, {"complex", 1})),
     OPT_CHOICE("ass-style-override", ass_style_override, 0,
-               ({"no", 0}, {"yes", 1}, {"force", 3})),
+               ({"no", 0}, {"yes", 1}, {"force", 3}, {"signfs", 4})),
     OPT_FLAG("sub-scale-with-window", sub_scale_with_window, 0),
     OPT_FLAG("osd-bar", osd_bar_visible, 0),
     OPT_FLOATRANGE("osd-bar-align-x", osd_bar_align_x, 0, -1.0, +1.0),
@@ -331,13 +351,17 @@ const m_option_t mp_opts[] = {
     OPT_FLOATRANGE("osd-bar-w", osd_bar_w, 0, 1, 100),
     OPT_FLOATRANGE("osd-bar-h", osd_bar_h, 0, 0.1, 50),
     OPT_SUBSTRUCT("osd", osd_style, osd_style_conf, 0),
+    OPT_FLAG("use-text-osd", use_text_osd, CONF_GLOBAL),
     OPT_SUBSTRUCT("sub-text", sub_text_style, osd_style_conf, 0),
+    OPT_FLAG("sub-clear-on-seek", sub_clear_on_seek, 0),
 
 //---------------------- libao/libvo options ------------------------
     OPT_SETTINGSLIST("vo", vo.video_driver_list, 0, &vo_obj_list),
     OPT_SETTINGSLIST("vo-defaults", vo.vo_defs, 0, &vo_obj_list),
     OPT_SETTINGSLIST("ao", audio_driver_list, 0, &ao_obj_list),
     OPT_SETTINGSLIST("ao-defaults", ao_defs, 0, &ao_obj_list),
+    OPT_STRING("audio-device", audio_device, 0),
+    OPT_STRING("audio-client-name", audio_client_name, 0),
     OPT_FLAG("fixed-vo", fixed_vo, CONF_GLOBAL),
     OPT_FLAG("force-window", force_vo, CONF_GLOBAL),
     OPT_FLAG("ontop", vo.ontop, M_OPT_FIXED),
@@ -350,7 +374,6 @@ const m_option_t mp_opts[] = {
                 {"yes", SOFTVOL_YES},
                 {"auto", SOFTVOL_AUTO})),
     OPT_FLOATRANGE("softvol-max", softvol_max, 0, 10, 10000),
-    OPT_INTRANGE("volstep", volstep, 0, 0, 100),
     OPT_FLOATRANGE("volume", mixer_init_volume, 0, -1, 100),
     OPT_CHOICE("mute", mixer_init_mute, M_OPT_OPTIONAL_PARAM,
                ({"auto", -1},
@@ -361,6 +384,8 @@ const m_option_t mp_opts[] = {
                ({"no", 0},
                 {"yes", 1}, {"", 1},
                 {"weak", -1})),
+    OPT_DOUBLE("audio-buffer", audio_buffer, M_OPT_MIN | M_OPT_MAX,
+               .min = 0, .max = 10),
 
     OPT_GEOMETRY("geometry", vo.geometry, 0),
     OPT_SIZE_BOX("autofit", vo.autofit, 0),
@@ -369,6 +394,7 @@ const m_option_t mp_opts[] = {
     // vo name (X classname) and window title strings
     OPT_STRING("x11-name", vo.winname, 0),
     OPT_STRING("title", wintitle, 0),
+    OPT_STRING("media-title", media_title, 0),
     // set aspect ratio of monitor - useful for 16:9 TV-out
     OPT_FLOATRANGE("monitoraspect", vo.force_monitor_aspect, 0, 0.0, 9.0),
     OPT_FLOATRANGE("monitorpixelaspect", vo.monitor_pixel_aspect, 0, 0.2, 9.0),
@@ -408,6 +434,7 @@ const m_option_t mp_opts[] = {
                 {"BT.2020", MP_CSP_PRIM_BT_2020})),
     OPT_CHOICE_OR_INT("video-rotate", video_rotate, 0, 0, 359,
                       ({"no", -1})),
+    OPT_VID_STEREO_MODE("video-stereo-mode", video_stereo_mode, 0),
 
     OPT_CHOICE_OR_INT("cursor-autohide", cursor_autohide_delay, 0,
                       0, 30000, ({"no", -1}, {"always", -2})),
@@ -416,7 +443,8 @@ const m_option_t mp_opts[] = {
 
     OPT_INT64("wid", vo.WinID, CONF_GLOBAL),
 #if HAVE_X11
-    OPT_FLAG("x11-netwm", vo.x11_netwm, 0),
+    OPT_CHOICE("x11-netwm", vo.x11_netwm, 0,
+               ({"auto", 0}, {"no", -1}, {"yes", 1})),
 #endif
     OPT_STRING("heartbeat-cmd", heartbeat_cmd, 0),
     OPT_FLOAT("heartbeat-interval", heartbeat_interval, CONF_MIN, 0),
@@ -427,16 +455,13 @@ const m_option_t mp_opts[] = {
     OPT_CHOICE_OR_INT("fs-screen", vo.fsscreen_id, 0, 0, 32,
                       ({"all", -2}, {"current", -1})),
 
-#if HAVE_COCOA
-    OPT_FLAG("fs-missioncontrol", vo.fs_missioncontrol, 0),
-#endif
-
     OPT_INTRANGE("brightness", gamma_brightness, 0, -100, 100),
     OPT_INTRANGE("saturation", gamma_saturation, 0, -100, 100),
     OPT_INTRANGE("contrast", gamma_contrast, 0, -100, 100),
     OPT_INTRANGE("hue", gamma_hue, 0, -100, 100),
     OPT_INTRANGE("gamma", gamma_gamma, 0, -100, 100),
     OPT_FLAG("keepaspect", vo.keepaspect, 0),
+    OPT_FLAG("keepaspect-window", vo.keepaspect_window, 0),
 
 //---------------------- mplayer-only options ------------------------
 
@@ -452,29 +477,38 @@ const m_option_t mp_opts[] = {
 
     OPT_CHOICE("framedrop", frame_dropping, 0,
                ({"no", 0},
-                {"yes", 1},
-                {"hard", 2})),
+                {"vo", 1},
+                {"decoder", 2},
+                {"decoder+vo", 3})),
+
+    OPT_DOUBLE("display-fps", frame_drop_fps, M_OPT_MIN, .min = 0),
 
     OPT_FLAG("untimed", untimed, M_OPT_FIXED),
 
-    OPT_STRING("stream-capture", stream_capture, M_OPT_FIXED),
-    OPT_STRING("stream-dump", stream_dump, M_OPT_FIXED),
+    OPT_STRING("stream-capture", stream_capture, M_OPT_FIXED | M_OPT_FILE),
+    OPT_STRING("stream-dump", stream_dump, M_OPT_FIXED | M_OPT_FILE),
+
+    OPT_FLAG("stop-playback-on-init-failure", stop_playback_on_init_failure, 0),
 
     OPT_CHOICE_OR_INT("loop", loop_times, M_OPT_GLOBAL, 2, 10000,
                       ({"no", -1}, {"1", -1},
                        {"inf", 0})),
-    OPT_FLAG("loop-file", loop_file, 0),
+    OPT_CHOICE_OR_INT("loop-file", loop_file, M_OPT_OPTIONAL_PARAM, 0, 10000,
+                      ({"yes", -1}, {"", -1}, {"no", 0}, // compat
+                       {"inf", -1})),
 
     OPT_FLAG("resume-playback", position_resume, 0),
     OPT_FLAG("save-position-on-quit", position_save_on_quit, 0),
     OPT_FLAG("write-filename-in-watch-later-config", write_filename_in_watch_later_config, 0),
 
     OPT_FLAG("ordered-chapters", ordered_chapters, 0),
-    OPT_STRING("ordered-chapters-files", ordered_chapters_files, 0),
+    OPT_STRING("ordered-chapters-files", ordered_chapters_files, M_OPT_FILE),
     OPT_INTRANGE("chapter-merge-threshold", chapter_merge_threshold, 0, 0, 10000),
 
     OPT_DOUBLE("chapter-seek-threshold", chapter_seek_threshold, 0),
 
+    OPT_STRING("chapters-file", chapter_file, M_OPT_FILE),
+
     OPT_FLAG("load-unsafe-playlists", load_unsafe_playlists, 0),
     OPT_FLAG("merge-files", merge_files, 0),
 
@@ -490,8 +524,6 @@ const m_option_t mp_opts[] = {
     OPT_CHOICE_OR_INT("autosync", autosync, 0, 0, 10000,
                       ({"no", -1})),
 
-    OPT_FLAG("softsleep", softsleep, 0),
-
     OPT_CHOICE("term-osd", term_osd, 0,
                ({"force", 1},
                 {"auto", 2},
@@ -501,13 +533,18 @@ const m_option_t mp_opts[] = {
     OPT_STRING("term-osd-bar-chars", term_osd_bar_chars, 0),
 
     OPT_STRING("term-playing-msg", playing_msg, 0),
+    OPT_STRING("osd-playing-msg", osd_playing_msg, 0),
     OPT_STRING("term-status-msg", status_msg, 0),
     OPT_STRING("osd-status-msg", osd_status_msg, 0),
+    OPT_STRING("osd-msg1", osd_msg[0], 0),
+    OPT_STRING("osd-msg2", osd_msg[1], 0),
+    OPT_STRING("osd-msg3", osd_msg[2], 0),
 
-    OPT_FLAG("slave-broken", slave_mode, CONF_GLOBAL),
     OPT_FLAG("idle", player_idle_mode, M_OPT_GLOBAL),
     OPT_FLAG("input-terminal", consolecontrols, CONF_GLOBAL),
-    OPT_FLAG("input-cursor", vo.enable_mouse_movements, CONF_GLOBAL),
+
+    OPT_STRING("input-file", input_file, M_OPT_FILE | M_OPT_GLOBAL),
+    OPT_STRING("input-unix-socket", ipc_path, M_OPT_FILE),
 
     OPT_SUBSTRUCT("screenshot", screenshot_image_opts, image_writer_conf, 0),
     OPT_STRING("screenshot-template", screenshot_template, 0),
@@ -515,6 +552,7 @@ const m_option_t mp_opts[] = {
     OPT_SUBSTRUCT("input", input_opts, input_config, 0),
 
     OPT_PRINT("list-properties", property_print_help),
+    OPT_PRINT("list-protocols", stream_print_proto_list),
     OPT_PRINT("help", print_help),
     OPT_PRINT("h", print_help),
     OPT_PRINT("version", print_version),
@@ -524,6 +562,8 @@ const m_option_t mp_opts[] = {
     OPT_SUBSTRUCT("", encode_opts, encode_config, 0),
 #endif
 
+    OPT_FLAG("slave-broken", slave_mode, CONF_GLOBAL),
+
     {0}
 };
 
@@ -539,19 +579,20 @@ const struct MPOpts mp_default_opts = {
     .softvol_max = 200,
     .mixer_init_volume = -1,
     .mixer_init_mute = -1,
-    .volstep = 3,
     .gapless_audio = -1,
+    .audio_buffer = 0.2,
+    .audio_device = "auto",
+    .audio_client_name = "mpv",
     .vo = {
         .video_driver_list = NULL,
         .monitor_pixel_aspect = 1.0,
         .screen_id = -1,
         .fsscreen_id = -1,
-        .enable_mouse_movements = 1,
         .panscan = 0.0f,
         .keepaspect = 1,
+        .keepaspect_window = 1,
         .border = 1,
         .WinID = -1,
-        .x11_netwm = 1,
     },
     .allow_win_drag = 1,
     .wintitle = "mpv - ${media-title}",
@@ -570,8 +611,11 @@ const struct MPOpts mp_default_opts = {
     .osd_bar_h = 3.125,
     .osd_scale = 1,
     .osd_scale_by_window = 1,
+    .use_text_osd = 1,
 #if HAVE_LUA
     .lua_load_osc = 1,
+    .lua_load_ytdl = 1,
+    .lua_ytdl_format = NULL,
 #endif
     .auto_load_scripts = 1,
     .loop_times = -1,
@@ -588,15 +632,21 @@ const struct MPOpts mp_default_opts = {
         .seek_min = 500,
         .file_max = 1024 * 1024,
     },
-    .stream_cache_pause = 50,
-    .stream_cache_unpause = 100,
+    .demuxer_thread = 1,
+    .demuxer_min_packs = 0,
+    .demuxer_min_bytes = 0,
+    .demuxer_min_secs = 0.2,
     .network_rtsp_transport = 2,
+    .demuxer_min_secs_cache = 2,
+    .cache_pausing = 1,
     .chapterrange = {-1, -1},
+    .ab_loop = {MP_NOPTS_VALUE, MP_NOPTS_VALUE},
     .edition_id = -1,
     .default_max_pts_correction = -1,
     .correct_pts = 1,
     .user_pts_assoc_mode = 1,
     .initial_audio_sync = 1,
+    .frame_dropping = 1,
     .term_osd = 2,
     .term_osd_bar_chars = "[-+-]",
     .consolecontrols = 1,
@@ -605,15 +655,18 @@ const struct MPOpts mp_default_opts = {
     .audio_id = -1,
     .video_id = -1,
     .sub_id = -1,
+    .audio_id_ff = -1,
+    .video_id_ff = -1,
+    .sub_id_ff = -1,
     .sub2_id = -2,
     .audio_display = 1,
     .sub_visibility = 1,
     .sub_pos = 100,
     .sub_speed = 1.0,
-    .quvi_fetch_subtitles = 0,
     .audio_output_channels = MP_CHMAP_INIT_STEREO,
     .audio_output_format = 0,  // AF_FORMAT_UNKNOWN
     .playback_speed = 1.,
+    .pitch_correction = 1,
     .movie_aspect = -1.,
     .field_dominance = -1,
     .sub_auto = 0,
@@ -629,11 +682,8 @@ const struct MPOpts mp_default_opts = {
     .ass_shaper = 1,
     .use_embedded_fonts = 1,
     .sub_fix_timing = 1,
-#if HAVE_ENCA
-    .sub_cp = "enca",
-#else
-    .sub_cp = "UTF-8:UTF-8-BROKEN",
-#endif
+    .sub_cp = "auto",
+    .mkv_subtitle_preroll_secs = 1.0,
 
     .hwdec_codecs = "h264,vc1,wmv3",
 
