@@ -1853,9 +1853,13 @@ function tick()
     end
 end
 
+function do_enable_keybindings()
+    mp.enable_key_bindings("showhide", "allow-vo-dragging|allow-hide-cursor")
+end
+
 function enable_osc(enable)
     if enable then
-        mp.enable_key_bindings("showhide")
+        do_enable_keybindings()
         show_osc()
     else
         hide_osc()
@@ -1876,12 +1880,21 @@ mp.observe_property("fullscreen", "bool", function(name, val) state.fullscreen =
 mp.observe_property("pause", "bool", pause_state)
 mp.observe_property("cache-idle", "bool", cache_state)
 
+mp.observe_property("disc-menu-active", "bool", function(name, val)
+    if val == true then
+        hide_osc()
+        mp.disable_key_bindings("showhide")
+    else
+        do_enable_keybindings()
+    end
+end)
+
 -- mouse show/hide bindings
 mp.set_key_bindings({
     {"mouse_move",              function(e) process_event("mouse_move", nil) end},
     {"mouse_leave",             mouse_leave},
-}, "showhide")
-mp.enable_key_bindings("showhide", "allow-vo-dragging|allow-hide-cursor")
+}, "showhide", "force")
+do_enable_keybindings()
 
 --mouse input bindings
 mp.set_key_bindings({
