@@ -119,6 +119,7 @@ const struct vd_lavc_hwdec mp_vd_lavc_vdpau;
 const struct vd_lavc_hwdec mp_vd_lavc_vda;
 const struct vd_lavc_hwdec mp_vd_lavc_vaapi;
 const struct vd_lavc_hwdec mp_vd_lavc_vaapi_copy;
+const struct vd_lavc_hwdec mp_vd_lavc_dxva2_copy;
 
 static const struct vd_lavc_hwdec *const hwdec_list[] = {
 #if HAVE_VDPAU_HWACCEL
@@ -130,6 +131,9 @@ static const struct vd_lavc_hwdec *const hwdec_list[] = {
 #if HAVE_VAAPI_HWACCEL
     &mp_vd_lavc_vaapi,
     &mp_vd_lavc_vaapi_copy,
+#endif
+#if HAVE_DXVA2_HWACCEL
+    &mp_vd_lavc_dxva2_copy,
 #endif
     NULL
 };
@@ -349,6 +353,8 @@ static void init_avctx(struct dec_video *vd, const char *decoder,
     ctx->hwdec_fmt = 0;
     ctx->avctx = avcodec_alloc_context3(lavc_codec);
     AVCodecContext *avctx = ctx->avctx;
+    if (!ctx->avctx)
+        goto error;
     avctx->bit_rate = 0;
     avctx->opaque = vd;
     avctx->codec_type = AVMEDIA_TYPE_VIDEO;
@@ -356,15 +362,15 @@ static void init_avctx(struct dec_video *vd, const char *decoder,
 
     avctx->refcounted_frames = 1;
     ctx->pic = av_frame_alloc();
+    if (!ctx->pic)
+        goto error;
 
     if (ctx->hwdec) {
         avctx->thread_count    = 1;
         avctx->get_format      = get_format_hwdec;
         avctx->get_buffer2     = get_buffer2_hwdec;
-        if (ctx->hwdec->init(ctx) < 0) {
-            uninit_avctx(vd);
-            return;
-        }
+        if (ctx->hwdec->init(ctx) < 0)
+            goto error;
     } else {
         mp_set_avcodec_threads(avctx, lavc_param->threads);
     }
@@ -414,11 +420,14 @@ static void init_avctx(struct dec_video *vd, const char *decoder,
         mp_copy_lav_codec_headers(avctx, sh->lav_headers);
 
     /* open it */
-    if (avcodec_open2(avctx, lavc_codec, NULL) < 0) {
-        MP_ERR(vd, "Could not open codec.\n");
-        uninit_avctx(vd);
-        return;
-    }
+    if (avcodec_open2(avctx, lavc_codec, NULL) < 0)
+        goto error;
+
+    return;
+
+error:
+    MP_ERR(vd, "Could not open codec.\n");
+    uninit_avctx(vd);
 }
 
 static void uninit_avctx(struct dec_video *vd)
@@ -426,6 +435,9 @@ static void uninit_avctx(struct dec_video *vd)
     vd_ffmpeg_ctx *ctx = vd->priv;
     AVCodecContext *avctx = ctx->avctx;
 
+    if (avctx && avctx->codec)
+        avcodec_flush_buffers(avctx);
+
     if (ctx->hwdec && ctx->hwdec->uninit)
         ctx->hwdec->uninit(ctx);
 
