@@ -141,6 +141,11 @@ static bool is_software_gl(GL *gl)
            strcmp(renderer, "Mesa X11") == 0;
 }
 
+static void GLAPIENTRY dummy_glBindFramebuffer(GLenum target, GLuint framebuffer)
+{
+    assert(framebuffer == 0);
+}
+
 #define FN_OFFS(name) offsetof(GL, name)
 
 #define DEF_FN(name)            {FN_OFFS(name), {"gl" # name}}
@@ -343,7 +348,7 @@ static const struct gl_functions gl_functions[] = {
     },
     {
         .extension = "GLX_SGI_video_sync",
-        .functions = (struct gl_function[]) {
+        .functions = (const struct gl_function[]) {
             DEF_FN_NAMES(GetVideoSync, "glXGetVideoSyncSGI"),
             DEF_FN_NAMES(WaitVideoSync, "glXWaitVideoSyncSGI"),
             {0},
@@ -618,6 +623,10 @@ void mpgl_load_functions(GL *gl, void *(*getProcAddress)(const GLubyte *),
 
     mp_verbose(log, "Detected OpenGL features:");
     list_features(gl->mpgl_caps, log, MSGL_V, false);
+
+    // Provided for simpler handling if no framebuffer support is available.
+    if (!gl->BindFramebuffer)
+        gl->BindFramebuffer = &dummy_glBindFramebuffer;
 }
 
 /**
@@ -879,6 +888,9 @@ static const struct backend backends[] = {
 #endif
 #if HAVE_GL_X11
     {"x11", mpgl_set_backend_x11},
+#endif
+#if HAVE_EGL_X11
+    {"x11egl", mpgl_set_backend_x11egl},
 #endif
     {0}
 };
