@@ -22,19 +22,23 @@ def __add_generic_flags__(ctx):
                        "-D_LARGEFILE64_SOURCE",
                        "-std=c99", "-Wall"]
 
+    if ctx.is_optimization():
+        ctx.env.CFLAGS += ['-O2']
+
     if ctx.is_debug_build():
         ctx.env.CFLAGS += ['-g']
 
 def __add_gcc_flags__(ctx):
     ctx.env.CFLAGS += ["-Wall", "-Wundef", "-Wmissing-prototypes", "-Wshadow",
                        "-Wno-switch", "-Wno-parentheses", "-Wpointer-arith",
-                       "-Wredundant-decls", "-Wno-pointer-sign",
-                       "-Werror=implicit-function-declaration",
-                       "-Wno-error=deprecated-declarations",
-                       "-Wno-error=unused-function" ]
-    __test_and_add_flags__(ctx, ["-Wempty-body"])
-    __test_and_add_flags__(ctx, ["-Wdisabled-optimization"])
-    __test_and_add_flags__(ctx, ["-Wstrict-prototypes"])
+                       "-Wredundant-decls", "-Wno-pointer-sign"]
+    __test_and_add_flags__(ctx, ["-Werror=implicit-function-declaration",
+                                 "-Wno-error=deprecated-declarations",
+                                 "-Wno-error=unused-function",
+                                 "-Wempty-body",
+                                 "-Wdisabled-optimization",
+                                 "-Wstrict-prototypes",
+                                 "-Wno-format-zero-length"])
 
 def __add_clang_flags__(ctx):
     ctx.env.CFLAGS += ["-Wno-logical-op-parentheses", "-fcolor-diagnostics",
@@ -43,9 +47,6 @@ def __add_clang_flags__(ctx):
 
 def __add_mingw_flags__(ctx):
     ctx.env.CFLAGS += ['-D__USE_MINGW_ANSI_STDIO=1']
-    ctx.env.CFLAGS += ['-DBYTE_ORDER=1234']
-    ctx.env.CFLAGS += ['-DLITLE_ENDIAN=1234']
-    ctx.env.CFLAGS += ['-DBIG_ENDIAN=4321']
     ctx.env.LAST_LINKFLAGS += ['-mwindows']
 
 def __add_cygwin_flags__(ctx):
